package netmap

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/pkg/morph/client"
)

// NewEpoch updates FrostFS epoch number through
// Netmap contract call.
// If `force` is true, this call is normally initiated by a control
// service command and uses a control notary transaction internally
// to ensure all nodes produce the same transaction with high probability.
func (c *Client) NewEpoch(epoch uint64, force bool) error {
	prm := client.InvokePrm{}
	prm.SetMethod(newEpochMethod)
	prm.SetArgs(epoch)
	prm.SetControlTX(force)

	if err := c.client.Invoke(prm); err != nil {
		return fmt.Errorf("could not invoke method (%s): %w", newEpochMethod, err)
	}
	return nil
}
